import * as acm from 'aws-cdk-lib/aws-certificatemanager';
import * as ec2 from 'aws-cdk-lib/aws-ec2';
import * as elbv2 from 'aws-cdk-lib/aws-elasticloadbalancingv2';
import * as route53 from 'aws-cdk-lib/aws-route53';
import * as s3 from 'aws-cdk-lib/aws-s3';
import { Construct } from 'constructs';
/**
 * Properties for PrivateS3Hosting
 */
export interface PrivateS3HostingProps {
    /**
     * The domain name for the website
     *
     * S3 bucket name will be created with `domainName`.
     *
     * If `enablePrivateDns` is enabled,
     * a private hosted zone also will be created for the `domainName`
     * and an A record has been created from `domainName` to the ALB DNS name.".
     *
     * If `subDomein` is provided, these names will be `${subDomain}.${domainName}`.
     */
    readonly domainName: string;
    /**
     * The sub domain for the website
     *
     * S3 bucket name will be created with `${subDomain}.{domainName}`.
     *
     * If `enablePrivateDns` is enabled,
     * a private hosted zone also will be created for the `domainName`
     * and an A record has been created from `${subDomain}.${domainName}` to the ALB DNS name.".
     *
     * @default - no sub domain
     */
    readonly subDomain?: string;
    /**
     * The certificate for the website
     *
     * @default - use HTTP
     */
    readonly certificate?: acm.ICertificate;
    /**
     * Enable private DNS for the website
     *
     * By eneabling this, a private hosted zone will be created for the domain name
     * and an alias record will be created for the ALB
     *
     * You can access to the alb by the `http(s)://<domainName>` from the VPC
     *
     * @default true
     */
    readonly enablePrivateDns?: boolean;
    /**
     * The properties for the S3 bucket
     *
     * @default - use default properties
     */
    readonly bucketProps: s3.BucketProps;
    /**
     * Whether the ALB is internet facing
     *
     * @default false
     */
    readonly internetFacing?: boolean;
    /**
     * The VPC for the website
     *
     * @default - create a new VPC with 2 AZs and 0 NAT gateways
     */
    readonly vpc?: ec2.IVpc;
}
/**
 * A construct to host a private S3 website
 */
export declare class PrivateS3Hosting extends Construct {
    /**
     * The S3 bucket for hosting the website
     */
    readonly bucket: s3.Bucket;
    /**
     * The ALB to access the website
     */
    readonly alb: elbv2.ApplicationLoadBalancer;
    /**
     * The VPC
     */
    readonly vpc: ec2.IVpc;
    /**
     * The hosted zone for the website
     */
    readonly hostedZone?: route53.IHostedZone;
    constructor(scope: Construct, id: string, props: PrivateS3HostingProps);
}
